import { Constants } from "./Constants.js";
import { PingCompensatedCharacter } from "./PingCompensatedCharacter.js";
export class Ranger extends PingCompensatedCharacter {
    ctype = "ranger";
    async fiveShot(target1, target2, target3, target4, target5) {
        if (!this.ready)
            throw "We aren't ready yet [fiveShot].";
        const attackStarted = new Promise((resolve, reject) => {
            const projectiles = [];
            const attackCheck = (data) => {
                if (data.attacker == this.id
                    && data.type == "5shot"
                    && (data.target == target1 || data.target == target2 || data.target == target3 || data.target == target4 || data.target == target5)) {
                    projectiles.push(data.pid);
                }
            };
            const failCheck = (data) => {
                if (typeof data == "object") {
                    if (data.response == "cooldown" && data.skill == "5shot") {
                        this.socket.off("action", attackCheck);
                        this.socket.off("game_response", failCheck);
                        this.socket.off("disappearing_text", failCheck2);
                        this.socket.off("eval", cooldownCheck);
                        reject(`fiveShot failed due to cooldown (ms: ${data.ms}).`);
                    }
                    else if (data.response == "no_mp" && data.place == "5shot") {
                        this.socket.off("action", attackCheck);
                        this.socket.off("game_response", failCheck);
                        this.socket.off("disappearing_text", failCheck2);
                        this.socket.off("eval", cooldownCheck);
                        reject("fiveShot failed due to insufficient MP.");
                    }
                }
            };
            const failCheck2 = (data) => {
                if (data.message == "NO HITS" && data.id == this.id) {
                    this.socket.off("action", attackCheck);
                    this.socket.off("game_response", failCheck);
                    this.socket.off("disappearing_text", failCheck2);
                    this.socket.off("eval", cooldownCheck);
                    resolve(projectiles); // We technically 5shot, we just didn't hit anything
                }
            };
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]5shot['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.off("action", attackCheck);
                    this.socket.off("game_response", failCheck);
                    this.socket.off("disappearing_text", failCheck2);
                    this.socket.off("eval", cooldownCheck);
                    resolve(projectiles);
                }
            };
            setTimeout(() => {
                this.socket.off("action", attackCheck);
                this.socket.off("game_response", failCheck);
                this.socket.off("disappearing_text", failCheck2);
                this.socket.off("eval", cooldownCheck);
                reject(`5shot timeout (${Constants.TIMEOUT}ms)`);
            }, Constants.TIMEOUT);
            this.socket.on("action", attackCheck);
            this.socket.on("game_response", failCheck);
            this.socket.on("disappearing_text", failCheck2);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", {
            ids: [target1, target2, target3, target4, target5],
            name: "5shot"
        });
        return attackStarted;
    }
    // NOTE: UNTESTED
    async fourFinger(target) {
        if (!this.ready)
            throw "We aren't ready yet [fourFinger].";
        // TODO: Check that the target is not a monster.
        const marked = new Promise((resolve, reject) => {
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]4fingers['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.off("eval", cooldownCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.off("eval", cooldownCheck);
                reject(`fourfinger timeout (${Constants.TIMEOUT}ms)`);
            }, Constants.TIMEOUT);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", {
            id: target,
            name: "4fingers"
        });
        return marked;
    }
    async huntersMark(target) {
        if (!this.ready)
            throw "We aren't ready yet [huntersMark].";
        const marked = new Promise((resolve, reject) => {
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]huntersmark['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.off("eval", cooldownCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.off("eval", cooldownCheck);
                reject(`huntersmark timeout (${Constants.TIMEOUT}ms)`);
            }, Constants.TIMEOUT);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", {
            id: target,
            name: "huntersmark"
        });
        return marked;
    }
    async piercingShot(target) {
        if (!this.ready)
            throw "We aren't ready yet [piercingShot].";
        if (this.G.skills.piercingshot.mp > this.mp)
            throw "Not enough MP to use piercingShot";
        const piercingShotStarted = new Promise((resolve, reject) => {
            let projectile;
            const attackCheck = (data) => {
                if (data.attacker == this.id
                    && data.type == "piercingshot"
                    && data.target == target) {
                    projectile = data.pid;
                }
            };
            const cooldownCheck = (data) => {
                // NOTE: This is `attack`, not `piercingshot`, because they share a cooldown.
                if (/skill_timeout\s*\(\s*['"]attack['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.off("action", attackCheck);
                    this.socket.off("eval", cooldownCheck);
                    resolve(projectile);
                }
            };
            setTimeout(() => {
                this.socket.off("action", attackCheck);
                this.socket.off("eval", cooldownCheck);
                reject(`piercingshot timeout (${Constants.TIMEOUT}ms)`);
            }, Constants.TIMEOUT);
            this.socket.on("action", attackCheck);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", { id: target, name: "piercingshot" });
        return piercingShotStarted;
    }
    // NOTE: UNTESTED
    async poisonArrow(target, poison = this.locateItem("poison")) {
        if (!this.ready)
            throw "We aren't ready yet [poisonArrow].";
        if (poison === undefined)
            throw "We need poison to use this skill.";
        const poisonArrowed = new Promise((resolve, reject) => {
            let projectile;
            const attackCheck = (data) => {
                if (data.attacker == this.id
                    && data.type == "poisonarrow"
                    && data.target == target) {
                    projectile = data.pid;
                }
            };
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]poisonarrow['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.off("action", attackCheck);
                    this.socket.off("eval", cooldownCheck);
                    resolve(projectile);
                }
            };
            setTimeout(() => {
                this.socket.off("action", attackCheck);
                this.socket.off("eval", cooldownCheck);
                reject(`poisonarrow timeout (${Constants.TIMEOUT}ms)`);
            }, Constants.TIMEOUT);
            this.socket.on("action", attackCheck);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", { id: target, name: "poisonarrow", num: poison });
        return poisonArrowed;
    }
    /**
     * TODO: Add a fail check for when we supershot an entity that doesn't exist (probably already killed)
     *
     * @param {string} target
     * @return {*}  {Promise<string>}
     * @memberof Ranger
     */
    async superShot(target) {
        if (!this.ready)
            throw "We aren't ready yet [superShot].";
        if (this.G.skills.supershot.mp > this.mp)
            throw "Not enough MP to use superShot";
        const superShotStarted = new Promise((resolve, reject) => {
            let projectile;
            const attackCheck = (data) => {
                if (data.attacker == this.id
                    && data.type == "supershot"
                    && data.target == target) {
                    projectile = data.pid;
                }
            };
            // TODO: Find a socket event for the fail
            // const failCheck = (data: GameResponseData) => {
            //     if(typeof data == "string") {
            //         if(data == "no_target") {
            //         }
            //     }
            // }
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]supershot['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.off("action", attackCheck);
                    this.socket.off("eval", cooldownCheck);
                    resolve(projectile);
                }
            };
            setTimeout(() => {
                this.socket.off("action", attackCheck);
                this.socket.off("eval", cooldownCheck);
                reject(`supershot timeout (${Constants.TIMEOUT}ms)`);
            }, Constants.TIMEOUT);
            this.socket.on("action", attackCheck);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", { id: target, name: "supershot" });
        return superShotStarted;
    }
    async threeShot(target1, target2, target3) {
        if (!this.ready)
            throw "We aren't ready yet [threeShot].";
        const attackStarted = new Promise((resolve, reject) => {
            const projectiles = [];
            const attackCheck = (data) => {
                if (data.attacker == this.id
                    && data.type == "3shot"
                    && (data.target == target1 || data.target == target2 || data.target == target3)) {
                    projectiles.push(data.pid);
                }
            };
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]3shot['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.off("action", attackCheck);
                    this.socket.off("game_response", failCheck);
                    this.socket.off("disappearing_text", failCheck2);
                    this.socket.off("eval", cooldownCheck);
                    resolve(projectiles);
                }
            };
            const failCheck = (data) => {
                if (typeof data == "object") {
                    if (data.response == "cooldown" && data.skill == "3shot") {
                        this.socket.off("action", attackCheck);
                        this.socket.off("game_response", failCheck);
                        this.socket.off("disappearing_text", failCheck2);
                        this.socket.off("eval", cooldownCheck);
                        reject(`threeShot failed due to cooldown (ms: ${data.ms}).`);
                    }
                    else if (data.response == "no_mp" && data.place == "3shot") {
                        this.socket.off("action", attackCheck);
                        this.socket.off("game_response", failCheck);
                        this.socket.off("disappearing_text", failCheck2);
                        this.socket.off("eval", cooldownCheck);
                        reject("threeShot failed due to insufficient MP.");
                    }
                }
            };
            const failCheck2 = (data) => {
                if (data.message == "NO HITS" && data.id == this.id) {
                    this.socket.off("action", attackCheck);
                    this.socket.off("game_response", failCheck);
                    this.socket.off("disappearing_text", failCheck2);
                    this.socket.off("eval", cooldownCheck);
                    resolve(projectiles); // We technically 3shot, we just didn't hit anything
                }
            };
            setTimeout(() => {
                this.socket.off("action", attackCheck);
                this.socket.off("game_response", failCheck);
                this.socket.off("disappearing_text", failCheck2);
                this.socket.off("eval", cooldownCheck);
                reject(`3shot timeout (${Constants.TIMEOUT}ms)`);
            }, Constants.TIMEOUT);
            this.socket.on("action", attackCheck);
            this.socket.on("game_response", failCheck);
            this.socket.on("disappearing_text", failCheck2);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", {
            ids: [target1, target2, target3],
            name: "3shot"
        });
        return attackStarted;
    }
}
//# sourceMappingURL=Ranger.js.map